//------------------------------------------------------------------------------------------------
//
//	 SG Craft - Base class for stargate computer interface tile entities
//
//------------------------------------------------------------------------------------------------

package gcewing.sg;

import net.minecraft.tileentity.*;
import gcewing.sg.SGAddressing.AddressingError;

public class SGInterfaceTE extends BaseTileEntity {

//	public SGBaseTE getBaseTE() {
//		return SGBaseTE.getBaseTE(this);
//	}

	public SGBaseTE getBaseTE() {
		return SGBaseTE.get(getWorldObj(), xCoord, yCoord + 1, zCoord);
	}
	
	// Signature is really prependArgs(Object..., Object[])
	
	public static Object[] prependArgs(Object... args) {
		int preLength = args.length - 1;
		Object[] post = (Object[])args[preLength];
		Object[] xargs = new Object[preLength + post.length];
		for (int i = 0; i < preLength; i++)
			xargs[i] = args[i];
		for (int i = 0; i < post.length; i++)
			xargs[preLength + i] = post[i];
		return xargs;
	}
	
	public void rebroadcastNetworkPacket(Object packet) {
	}

	public static class CIStargateState {
		public String state;
		public int chevrons;
		public CIStargateState(String state, int chevrons) {
			this.state = state;
			this.chevrons = chevrons;
		}
	}
		
	public SGBaseTE requireBaseTE() {
		SGBaseTE te = getBaseTE();
		if (te != null && te.isMerged)
			return te;
		throw new IllegalArgumentException("No stargate connected to interface");
	}
	
	public SGBaseTE requireIrisTE() {
		SGBaseTE te = requireBaseTE();
		if (te != null && te.hasIrisUpgrade)
			return te;
		else
			throw new IllegalArgumentException("No iris fitted to stargate");
	}
	
	public CIStargateState ciStargateState() {
		SGBaseTE te = getBaseTE();
		if (te != null)
			return new CIStargateState(te.sgStateDescription(), te.numEngagedChevrons);
		else
			return new CIStargateState("Offline", 0);
	}
	
	public double ciEnergyAvailable() {
		SGBaseTE te = getBaseTE();
		if (te != null)
			return te.availableEnergy();
		else
			return 0;
	}
	
	public double ciEnergyToDial(String address) {
		SGBaseTE te = requireBaseTE();
		try {
			address = SGAddressing.normalizeAddress(address);
			SGBaseTE dte = SGAddressing.findAddressedStargate(address, te.getWorldObj());
			if (dte == null)
				throw new IllegalArgumentException("No stargate at address " + address);
			double distanceFactor = SGBaseTE.distanceFactorForCoordDifference(te, dte);
			return SGBaseTE.energyToOpen * distanceFactor;
		}
		catch (AddressingError e) {
			System.out.printf("SGBaseTE.ciEnergyToDial: caught %s\n", e);
			throw new IllegalArgumentException(e.getMessage());
		}
	}
	
	public String ciLocalAddress() {
		SGBaseTE te = getBaseTE();
		try {
			if (te != null)
				return te.getHomeAddress();
			else
				return "";
		}
		catch (AddressingError e) {
			throw new IllegalArgumentException(e.getMessage());
		}
	}
	
	public String ciRemoteAddress() {
		SGBaseTE te = requireBaseTE();
		try {
			if (te.connectedLocation != null)
				return SGAddressing.addressForLocation(te.connectedLocation);
			else
				return "";
		}
		catch (AddressingError e) {
			throw new IllegalArgumentException(e.getMessage());
		}
	}
	
	public void ciDial(String address) {
		SGBaseTE te = requireBaseTE();
		try {
			address = SGAddressing.normalizedRelativeAddress(address, te.getHomeAddress());
			System.out.printf("SGBaseTE.ciDial: dialling symbols %s\n", address);
			String error = te.connect(address, null);
			if (error != null)
				throw new IllegalArgumentException(error);
		}
		catch (AddressingError e) {
			throw new IllegalArgumentException(e.getMessage());
		}
	}
	
	public void ciDisconnect() {
		requireBaseTE().disconnect();
	}
	
	public String ciIrisState() {
		SGBaseTE te = getBaseTE();
		if (te != null && te.hasIrisUpgrade)
			return te.irisStateDescription();
		else
			return "Offline";
	}
	
	public void ciOpenIris() {
		requireIrisTE().openIris();
	}
	
	public void ciCloseIris() {
		requireIrisTE().closeIris();
	}
	
	public void ciSendMessage(Object[] args) {
		SGBaseTE te = getBaseTE();
		if (te != null)
			te.sendMessage(args);
	}

}
